!(function(global) {
	var $mipm = global.$mipm;// native 对象
	var $isAndroid = (/android/gi).test(navigator.appVersion);
	// CMD规范模型，包含 api 调用时 callback 存储对象，callback id生成器
	var $mipm_cmd_obj = {
		count: 1,
		fn: {},
		put: function(id, cb){
			this.fn[id] = cb;
		},
		id: function(){
			return this.count++;
		},
		on: function(cbId, ret, err, del) {
			this.fn[cbId] && this.fn[cbId](ret, err);
			del && this.gc(cbId);
		},
		gc: function(cbId) {
			delete this.fn[cbId];
		}
	}
	// 转换 api 调用，存储 callback 至 $mipm_cmd_obj 对象，并返回序列化后的传参
	var $callCover = function(p) {
		var op = {};
		if (p.length === 1) {
			if (typeof p[0] === 'function') {
				op.cbId = $mipm_cmd_obj.id();
				$mipm_cmd_obj.put(op.cbId, p[0]);
			} else {
				op.arg = p[0];
			}
		} else if (p.length === 2) {
			op.arg = p[0];
			op.cbId = $mipm_cmd_obj.id();
			$mipm_cmd_obj.put(op.cbId, p[1]);
		}
		return JSON.stringify(op);
	};
	// 还原原生返回结果为真实数据
	var $callbackCover = function(m) {
		if(!m){
			return m;
		}
		if (m.startsWith('[s')) {return m.slice(2);
		} else if (m.startsWith('[t')) {return true;
		} else if (m.startsWith('[f')) {return false;
		} else if (m.startsWith('[N')) {return null;
		} else if (m.startsWith('[n')) {return +m.slice(2);
		} else if (m.startsWith('[J')) {return JSON.parse(m.slice(2));
		}
		return m;
	};
	// native 回调 js 分发对象
	var mipm$cb = {
		// native callback
		callback: function(cbId, ret, err, del) {
			$mipm_cmd_obj.on(cbId, ret, err, del);
		},
		// native 主动回收 js 缓存的 callback 函数
		gc: function(cbId) {
			$mipm_cmd_obj.gc(cbId);
		}
	}

	var obj_mipm = {};

	function $(id){
		return document.getElementById(id);
	}
	obj_mipm.$ = $;

	function $n(n){
		return document.getElementsByName(n);
	}
	obj_mipm.$n = $n;

	// alert走原生实现，异步
	obj_mipm.alert = function (_msg, cb){
		if(!$mipm){
			return global.alert(_msg);
		}
		$nativeMethod('alert', {msg: _msg}, cb);
	}
	// confirm 走js的同步，清理缓存界面有用到
	obj_mipm.confirm = function (_msg, cb){
		//if(!$mipm){
			return global.confirm(_msg);
		//}
		//$nativeMethod('confirm', {msg: _msg}, cb);
	}

	obj_mipm.toast = function (_msg){
		if(!$mipm){
			global.alert('toast: ' + _msg);
			return;
		}
		$nativeMethod('toast', {msg: _msg});
	}

	obj_mipm.open = function(viewId, args){
		if($mipm){
			args = args ? JSON.stringify(args) : "{}";
			$mipm.open(viewId, args);
			return;
		}
		var url = './html/' + viewId + '.html';
		if(global.api){
			global.api.openWin({
				name: viewId,
				url: url
			});
		}else{
			global.location.href = url;
		}
	}

	obj_mipm.close = function(viewId, args){
		if($mipm){
			args = args ? JSON.stringify(args) : "{}";
			$mipm.close(viewId, args);
			return;
		}
		if(global.history.length > 0){
			global.history.back();
			return;
		}
		if(global.api){
			global.api.closeWin();
		}else{
			console.error('no bridge for "mipm.close".');
		}
	}

	obj_mipm.getViewData = function(viewId, args){
		viewId = viewId ? viewId : 'index';
		if($mipm){
			args = args ? JSON.stringify(args) : "{}";
			var str = $mipm.getViewData(viewId, args);
			return str ? JSON.parse(str) : {};
		}
		console.log(viewId + ' run in mock data. ');
		return global.mockData ? global.mockData[viewId] : {};
	}

	obj_mipm.syncData = function(args, cb){
		$nativeMethod('syncData', args, cb);
	}

	obj_mipm.exportData = function(args, cb){
		$nativeMethod('exportData', args, cb);
	}

	obj_mipm.shareData = function(args, cb){
		$nativeMethod('shareData', args, cb);
	}

	obj_mipm.getSyncDate = function(args, cb){
		return $nativeMethod('getSyncDate', args, cb);
	}

	obj_mipm.getSyncTime = function(args, cb){
		return $nativeMethod('getSyncTime', args, cb);
	}

	obj_mipm.cleanCache = function(args, cb){
		$nativeMethod('cleanCache', args, cb);
	}

	obj_mipm.cleanData = function(args, cb){
		$nativeMethod('cleanData', args, cb);
	}

	obj_mipm.capacity = function(args, cb){
		return $nativeMethod('capacity', args, cb);
	}

	var $nativeMethod = function(name, args, cb){
		if($mipm){
			if(0 == arguments.length){
				console.warn('no bridge name for "mipm.method".');
				return;
			}
			var name = arguments[0];
			var ops = [];
			if(arguments.length > 1){
				for(var i = 1; i < arguments.length; ++i){
					ops.push(arguments[i]);
				}
			}
			var op = $callCover(ops);
			return $callbackCover($mipm.callMethod(name, op));
		}
		console.warn('no bridge for "mipm.method".');
	}

	global.$ = $;
	global.$n = $n;
	global.api$dispatcher = mipm$cb;
	global.toast = obj_mipm.toast;
	global.mipm = obj_mipm;
	global.isAndroid = $isAndroid;
	
})(window);
